/*
 * gen68k.c: Genital68K: Genital's custom 68K emulator
 */

#include "gen68k.h"


/*****************************************************************************
* Macros                                                                    */

#if GENITAL68K_ADDR == 24
        #define ADDR_CLIP(addr) addr &= 0xffffff;
#elif GENITAL68K_ADDR == 32
        #define ADDR_CLIP(addr) addr &= 0xffffffff;
#else   /* 24 is default */
        #define ADDR_CLIP(addr) addr &= 0xffffff;
#endif


#define CYCLES(i) cpu->timer -= i;


/*****************************************************************************
* Context data                                                              */

struct GENITAL68K_CPUCONTEXT    genital68kcontext;
struct GENITAL68K_CPUCONTEXT    *cpu;


/*****************************************************************************
* Memory access handlers                                                    */

UINT32 ReadByte(UINT32 addr)
{
        int     i = 0;
        ADDR_CLIP(addr);
        while (cpu->read_byte[i].base != -1)
        {                       
                if ((addr >= cpu->read_byte[i].base) && (addr <= cpu->read_byte[i].limit))
                        return cpu->read_byte[i].handler(addr);
                i++;
        }
        return 0;
}

UINT32 ReadWord(UINT32 addr)
{
        int     i = 0;
        ADDR_CLIP(addr);
        while (cpu->read_word[i].base != -1)
        {
                if ((addr >= cpu->read_word[i].base) && (addr <= cpu->read_word[i].limit))
                        return cpu->read_word[i].handler(addr);
                i++;
        }
        return 0;
}

UINT32 ReadLong(UINT32 addr)
{
        return (ReadWord(addr) << 16) | ReadWord(addr + 2);
}

void WriteByte(UINT32 addr, UINT32 data)
{
        int     i = 0;
        ADDR_CLIP(addr);
        while (cpu->write_byte[i].base != -1)
        {
                if ((addr >= cpu->write_byte[i].base) && (addr <= cpu->write_byte[i].limit))
                {
                        cpu->write_byte[i].handler(addr, data & 0xff);
                        return;
                }
                i++;
        }
}

void WriteWord(UINT32 addr, UINT32 data)
{
        int     i = 0;
        ADDR_CLIP(addr);
        while (cpu->write_word[i].base != -1)
        {
                if ((addr >= cpu->write_word[i].base) && (addr <= cpu->write_word[i].limit))
                {
                        cpu->write_word[i].handler(addr, data & 0xff);
                        return;
                }
                i++;
        }
}               

void WriteLong(UINT32 addr, UINT32 data)
{
        WriteWord(addr, (data >> 16) & 0xffff);
        WriteWord(addr + 2, data & 0xffff);
}


/*****************************************************************************
* External access to registers                                              */

UINT32 Genital68KReadPC()
{
        return cpu->pc;
}

UINT32 Genital68KReadSP()
{
        return cpu->sp;
}

UINT32 Genital68KReadSR()
{
        return cpu->sr;
}

UINT32 Genital68KReadA(int i)
{
        return cpu->a[i & 7];
}

UINT32 Genital68KReadD(int i)
{
        return cpu->d[i & 7];
}


/*****************************************************************************
* Context management                                                        */

struct GENITAL68K_CPUCONTEXT *Genital68KGetContext()
{
        return cpu;
}

void Genital68KSetContext(struct GENITAL68K_CPUCONTEXT *new)
{
        cpu = new;
}


/*****************************************************************************
* Major emulator functions                                                  */

int Genital68KInit(struct GENITAL68K_CPUCONTEXT *context)
{
        Genital68KSetContext(context);
        return 0;
}

void Genital68KReset()
{
        int     i;

        for (i = 0; i < 8; i++)  
        {
                cpu->a[i] = 0;
                cpu->d[i] = 0;
        }
        cpu->sr = 0;
        cpu->a[7] = ReadLong(0);        /* initial stack pointer */
        cpu->pc = ReadLong(4);          /* initial PC */
}

int Genital68KRun(int cycles)
{
        cpu->timer = cycles;

//        while (cycles > 0)
//                Jump(FetchOp())[FetchOp()];

        return cycles - cpu->timer;
}


/*****************************************************************************
* Instruction handlers                                                      */

void InstINVALID(UINT32 pc, UINT32 op)
{
        CYCLES(2);
}

void InstMOVE_Word(UINT32 pc, UINT32 op)
{
}

/*****************************************************************************
* Jump table                                                                 */

/*
 * Table of 1024 entries, (opcode >> 6) will yield an index into this table
 */

typedef void (*inst_handler)(UINT32, UINT32);
inst_handler jump[1024] = {
/* 000 */	InstINVALID,
/* 001 */	InstINVALID,
/* 002 */	InstINVALID,
/* 003 */	InstINVALID,
/* 004 */	InstINVALID,
/* 005 */	InstINVALID,
/* 006 */	InstINVALID,
/* 007 */	InstINVALID,
/* 008 */	InstINVALID,
/* 009 */	InstINVALID,
/* 00A */	InstINVALID,
/* 00B */	InstINVALID,
/* 00C */	InstINVALID,
/* 00D */	InstINVALID,
/* 00E */	InstINVALID,
/* 00F */	InstINVALID,
/* 010 */	InstINVALID,
/* 011 */	InstINVALID,
/* 012 */	InstINVALID,
/* 013 */	InstINVALID,
/* 014 */	InstINVALID,
/* 015 */	InstINVALID,
/* 016 */	InstINVALID,
/* 017 */	InstINVALID,
/* 018 */	InstINVALID,
/* 019 */	InstINVALID,
/* 01A */	InstINVALID,
/* 01B */	InstINVALID,
/* 01C */	InstINVALID,
/* 01D */	InstINVALID,
/* 01E */	InstINVALID,
/* 01F */	InstINVALID,
/* 020 */	InstINVALID,
/* 021 */	InstINVALID,
/* 022 */	InstINVALID,
/* 023 */	InstINVALID,
/* 024 */	InstINVALID,
/* 025 */	InstINVALID,
/* 026 */	InstINVALID,
/* 027 */	InstINVALID,
/* 028 */	InstINVALID,
/* 029 */	InstINVALID,
/* 02A */	InstINVALID,
/* 02B */	InstINVALID,
/* 02C */	InstINVALID,
/* 02D */	InstINVALID,
/* 02E */	InstINVALID,
/* 02F */	InstINVALID,
/* 030 */	InstINVALID,
/* 031 */	InstINVALID,
/* 032 */	InstINVALID,
/* 033 */	InstINVALID,
/* 034 */	InstINVALID,
/* 035 */	InstINVALID,
/* 036 */	InstINVALID,
/* 037 */	InstINVALID,
/* 038 */	InstINVALID,
/* 039 */	InstINVALID,
/* 03A */	InstINVALID,
/* 03B */	InstINVALID,
/* 03C */	InstINVALID,
/* 03D */	InstINVALID,
/* 03E */	InstINVALID,
/* 03F */	InstINVALID,
/* 040 */	InstINVALID,
/* 041 */	InstINVALID,
/* 042 */	InstINVALID,
/* 043 */	InstINVALID,
/* 044 */	InstINVALID,
/* 045 */	InstINVALID,
/* 046 */	InstINVALID,
/* 047 */	InstINVALID,
/* 048 */	InstINVALID,
/* 049 */	InstINVALID,
/* 04A */	InstINVALID,
/* 04B */	InstINVALID,
/* 04C */	InstINVALID,
/* 04D */	InstINVALID,
/* 04E */	InstINVALID,
/* 04F */	InstINVALID,
/* 050 */	InstINVALID,
/* 051 */	InstINVALID,
/* 052 */	InstINVALID,
/* 053 */	InstINVALID,
/* 054 */	InstINVALID,
/* 055 */	InstINVALID,
/* 056 */	InstINVALID,
/* 057 */	InstINVALID,
/* 058 */	InstINVALID,
/* 059 */	InstINVALID,
/* 05A */	InstINVALID,
/* 05B */	InstINVALID,
/* 05C */	InstINVALID,
/* 05D */	InstINVALID,
/* 05E */	InstINVALID,
/* 05F */	InstINVALID,
/* 060 */	InstINVALID,
/* 061 */	InstINVALID,
/* 062 */	InstINVALID,
/* 063 */	InstINVALID,
/* 064 */	InstINVALID,
/* 065 */	InstINVALID,
/* 066 */	InstINVALID,
/* 067 */	InstINVALID,
/* 068 */	InstINVALID,
/* 069 */	InstINVALID,
/* 06A */	InstINVALID,
/* 06B */	InstINVALID,
/* 06C */	InstINVALID,
/* 06D */	InstINVALID,
/* 06E */	InstINVALID,
/* 06F */	InstINVALID,
/* 070 */	InstINVALID,
/* 071 */	InstINVALID,
/* 072 */	InstINVALID,
/* 073 */	InstINVALID,
/* 074 */	InstINVALID,
/* 075 */	InstINVALID,
/* 076 */	InstINVALID,
/* 077 */	InstINVALID,
/* 078 */	InstINVALID,
/* 079 */	InstINVALID,
/* 07A */	InstINVALID,
/* 07B */	InstINVALID,
/* 07C */	InstINVALID,
/* 07D */	InstINVALID,
/* 07E */	InstINVALID,
/* 07F */	InstINVALID,
/* 080 */	InstINVALID,
/* 081 */	InstINVALID,
/* 082 */	InstINVALID,
/* 083 */	InstINVALID,
/* 084 */	InstINVALID,
/* 085 */	InstINVALID,
/* 086 */	InstINVALID,
/* 087 */	InstINVALID,
/* 088 */	InstINVALID,
/* 089 */	InstINVALID,
/* 08A */	InstINVALID,
/* 08B */	InstINVALID,
/* 08C */	InstINVALID,
/* 08D */	InstINVALID,
/* 08E */	InstINVALID,
/* 08F */	InstINVALID,
/* 090 */	InstINVALID,
/* 091 */	InstINVALID,
/* 092 */	InstINVALID,
/* 093 */	InstINVALID,
/* 094 */	InstINVALID,
/* 095 */	InstINVALID,
/* 096 */	InstINVALID,
/* 097 */	InstINVALID,
/* 098 */	InstINVALID,
/* 099 */	InstINVALID,
/* 09A */	InstINVALID,
/* 09B */	InstINVALID,
/* 09C */	InstINVALID,
/* 09D */	InstINVALID,
/* 09E */	InstINVALID,
/* 09F */	InstINVALID,
/* 0A0 */	InstINVALID,
/* 0A1 */	InstINVALID,
/* 0A2 */	InstINVALID,
/* 0A3 */	InstINVALID,
/* 0A4 */	InstINVALID,
/* 0A5 */	InstINVALID,
/* 0A6 */	InstINVALID,
/* 0A7 */	InstINVALID,
/* 0A8 */	InstINVALID,
/* 0A9 */	InstINVALID,
/* 0AA */	InstINVALID,
/* 0AB */	InstINVALID,
/* 0AC */	InstINVALID,
/* 0AD */	InstINVALID,
/* 0AE */	InstINVALID,
/* 0AF */	InstINVALID,
/* 0B0 */	InstINVALID,
/* 0B1 */	InstINVALID,
/* 0B2 */	InstINVALID,
/* 0B3 */	InstINVALID,
/* 0B4 */	InstINVALID,
/* 0B5 */	InstINVALID,
/* 0B6 */	InstINVALID,
/* 0B7 */	InstINVALID,
/* 0B8 */	InstINVALID,
/* 0B9 */	InstINVALID,
/* 0BA */	InstINVALID,
/* 0BB */	InstINVALID,
/* 0BC */	InstINVALID,
/* 0BD */	InstINVALID,
/* 0BE */	InstINVALID,
/* 0BF */	InstINVALID,
/* 0C0 */	InstINVALID,
/* 0C1 */	InstINVALID,
/* 0C2 */	InstINVALID,
/* 0C3 */	InstINVALID,
/* 0C4 */	InstINVALID,
/* 0C5 */	InstINVALID,
/* 0C6 */	InstINVALID,
/* 0C7 */	InstINVALID,
/* 0C8 */	InstINVALID,
/* 0C9 */	InstINVALID,
/* 0CA */	InstINVALID,
/* 0CB */	InstINVALID,
/* 0CC */	InstINVALID,
/* 0CD */	InstINVALID,
/* 0CE */	InstINVALID,
/* 0CF */	InstINVALID,
/* 0D0 */	InstINVALID,
/* 0D1 */	InstINVALID,
/* 0D2 */	InstINVALID,
/* 0D3 */	InstINVALID,
/* 0D4 */	InstINVALID,
/* 0D5 */	InstINVALID,
/* 0D6 */	InstINVALID,
/* 0D7 */	InstINVALID,
/* 0D8 */	InstINVALID,
/* 0D9 */	InstINVALID,
/* 0DA */	InstINVALID,
/* 0DB */	InstINVALID,
/* 0DC */	InstINVALID,
/* 0DD */	InstINVALID,
/* 0DE */	InstINVALID,
/* 0DF */	InstINVALID,
/* 0E0 */	InstINVALID,
/* 0E1 */	InstINVALID,
/* 0E2 */	InstINVALID,
/* 0E3 */	InstINVALID,
/* 0E4 */	InstINVALID,
/* 0E5 */	InstINVALID,
/* 0E6 */	InstINVALID,
/* 0E7 */	InstINVALID,
/* 0E8 */	InstINVALID,
/* 0E9 */	InstINVALID,
/* 0EA */	InstINVALID,
/* 0EB */	InstINVALID,
/* 0EC */	InstINVALID,
/* 0ED */	InstINVALID,
/* 0EE */	InstINVALID,
/* 0EF */	InstINVALID,
/* 0F0 */	InstINVALID,
/* 0F1 */	InstINVALID,
/* 0F2 */	InstINVALID,
/* 0F3 */	InstINVALID,
/* 0F4 */	InstINVALID,
/* 0F5 */	InstINVALID,
/* 0F6 */	InstINVALID,
/* 0F7 */	InstINVALID,
/* 0F8 */	InstINVALID,
/* 0F9 */	InstINVALID,
/* 0FA */	InstINVALID,
/* 0FB */	InstINVALID,
/* 0FC */	InstINVALID,
/* 0FD */	InstINVALID,
/* 0FE */	InstINVALID,
/* 0FF */	InstINVALID,
/* 100 */	InstINVALID,
/* 101 */	InstINVALID,
/* 102 */	InstINVALID,
/* 103 */	InstINVALID,
/* 104 */	InstINVALID,
/* 105 */	InstINVALID,
/* 106 */	InstINVALID,
/* 107 */	InstINVALID,
/* 108 */	InstINVALID,
/* 109 */	InstINVALID,
/* 10A */	InstINVALID,
/* 10B */	InstINVALID,
/* 10C */	InstINVALID,
/* 10D */	InstINVALID,
/* 10E */	InstINVALID,
/* 10F */	InstINVALID,
/* 110 */	InstINVALID,
/* 111 */	InstINVALID,
/* 112 */	InstINVALID,
/* 113 */	InstINVALID,
/* 114 */	InstINVALID,
/* 115 */	InstINVALID,
/* 116 */	InstINVALID,
/* 117 */	InstINVALID,
/* 118 */	InstINVALID,
/* 119 */	InstINVALID,
/* 11A */	InstINVALID,
/* 11B */	InstINVALID,
/* 11C */	InstINVALID,
/* 11D */	InstINVALID,
/* 11E */	InstINVALID,
/* 11F */	InstINVALID,
/* 120 */	InstINVALID,
/* 121 */	InstINVALID,
/* 122 */	InstINVALID,
/* 123 */	InstINVALID,
/* 124 */	InstINVALID,
/* 125 */	InstINVALID,
/* 126 */	InstINVALID,
/* 127 */	InstINVALID,
/* 128 */	InstINVALID,
/* 129 */	InstINVALID,
/* 12A */	InstINVALID,
/* 12B */	InstINVALID,
/* 12C */	InstINVALID,
/* 12D */	InstINVALID,
/* 12E */	InstINVALID,
/* 12F */	InstINVALID,
/* 130 */	InstINVALID,
/* 131 */	InstINVALID,
/* 132 */	InstINVALID,
/* 133 */	InstINVALID,
/* 134 */	InstINVALID,
/* 135 */	InstINVALID,
/* 136 */	InstINVALID,
/* 137 */	InstINVALID,
/* 138 */	InstINVALID,
/* 139 */	InstINVALID,
/* 13A */	InstINVALID,
/* 13B */	InstINVALID,
/* 13C */	InstINVALID,
/* 13D */	InstINVALID,
/* 13E */	InstINVALID,
/* 13F */	InstINVALID,
/* 140 */	InstINVALID,
/* 141 */	InstINVALID,
/* 142 */	InstINVALID,
/* 143 */	InstINVALID,
/* 144 */	InstINVALID,
/* 145 */	InstINVALID,
/* 146 */	InstINVALID,
/* 147 */	InstINVALID,
/* 148 */	InstINVALID,
/* 149 */	InstINVALID,
/* 14A */	InstINVALID,
/* 14B */	InstINVALID,
/* 14C */	InstINVALID,
/* 14D */	InstINVALID,
/* 14E */	InstINVALID,
/* 14F */	InstINVALID,
/* 150 */	InstINVALID,
/* 151 */	InstINVALID,
/* 152 */	InstINVALID,
/* 153 */	InstINVALID,
/* 154 */	InstINVALID,
/* 155 */	InstINVALID,
/* 156 */	InstINVALID,
/* 157 */	InstINVALID,
/* 158 */	InstINVALID,
/* 159 */	InstINVALID,
/* 15A */	InstINVALID,
/* 15B */	InstINVALID,
/* 15C */	InstINVALID,
/* 15D */	InstINVALID,
/* 15E */	InstINVALID,
/* 15F */	InstINVALID,
/* 160 */	InstINVALID,
/* 161 */	InstINVALID,
/* 162 */	InstINVALID,
/* 163 */	InstINVALID,
/* 164 */	InstINVALID,
/* 165 */	InstINVALID,
/* 166 */	InstINVALID,
/* 167 */	InstINVALID,
/* 168 */	InstINVALID,
/* 169 */	InstINVALID,
/* 16A */	InstINVALID,
/* 16B */	InstINVALID,
/* 16C */	InstINVALID,
/* 16D */	InstINVALID,
/* 16E */	InstINVALID,
/* 16F */	InstINVALID,
/* 170 */	InstINVALID,
/* 171 */	InstINVALID,
/* 172 */	InstINVALID,
/* 173 */	InstINVALID,
/* 174 */	InstINVALID,
/* 175 */	InstINVALID,
/* 176 */	InstINVALID,
/* 177 */	InstINVALID,
/* 178 */	InstINVALID,
/* 179 */	InstINVALID,
/* 17A */	InstINVALID,
/* 17B */	InstINVALID,
/* 17C */	InstINVALID,
/* 17D */	InstINVALID,
/* 17E */	InstINVALID,
/* 17F */	InstINVALID,
/* 180 */	InstINVALID,
/* 181 */	InstINVALID,
/* 182 */	InstINVALID,
/* 183 */	InstINVALID,
/* 184 */	InstINVALID,
/* 185 */	InstINVALID,
/* 186 */	InstINVALID,
/* 187 */	InstINVALID,
/* 188 */	InstINVALID,
/* 189 */	InstINVALID,
/* 18A */	InstINVALID,
/* 18B */	InstINVALID,
/* 18C */	InstINVALID,
/* 18D */	InstINVALID,
/* 18E */	InstINVALID,
/* 18F */	InstINVALID,
/* 190 */	InstINVALID,
/* 191 */	InstINVALID,
/* 192 */	InstINVALID,
/* 193 */	InstINVALID,
/* 194 */	InstINVALID,
/* 195 */	InstINVALID,
/* 196 */	InstINVALID,
/* 197 */	InstINVALID,
/* 198 */	InstINVALID,
/* 199 */	InstINVALID,
/* 19A */	InstINVALID,
/* 19B */	InstINVALID,
/* 19C */	InstINVALID,
/* 19D */	InstINVALID,
/* 19E */	InstINVALID,
/* 19F */	InstINVALID,
/* 1A0 */	InstINVALID,
/* 1A1 */	InstINVALID,
/* 1A2 */	InstINVALID,
/* 1A3 */	InstINVALID,
/* 1A4 */	InstINVALID,
/* 1A5 */	InstINVALID,
/* 1A6 */	InstINVALID,
/* 1A7 */	InstINVALID,
/* 1A8 */	InstINVALID,
/* 1A9 */	InstINVALID,
/* 1AA */	InstINVALID,
/* 1AB */	InstINVALID,
/* 1AC */	InstINVALID,
/* 1AD */	InstINVALID,
/* 1AE */	InstINVALID,
/* 1AF */	InstINVALID,
/* 1B0 */	InstINVALID,
/* 1B1 */	InstINVALID,
/* 1B2 */	InstINVALID,
/* 1B3 */	InstINVALID,
/* 1B4 */	InstINVALID,
/* 1B5 */	InstINVALID,
/* 1B6 */	InstINVALID,
/* 1B7 */	InstINVALID,
/* 1B8 */	InstINVALID,
/* 1B9 */	InstINVALID,
/* 1BA */	InstINVALID,
/* 1BB */	InstINVALID,
/* 1BC */	InstINVALID,
/* 1BD */	InstINVALID,
/* 1BE */	InstINVALID,
/* 1BF */	InstINVALID,
/* 1C0 */	InstINVALID,
/* 1C1 */	InstINVALID,
/* 1C2 */	InstINVALID,
/* 1C3 */	InstINVALID,
/* 1C4 */	InstINVALID,
/* 1C5 */	InstINVALID,
/* 1C6 */	InstINVALID,
/* 1C7 */	InstINVALID,
/* 1C8 */	InstINVALID,
/* 1C9 */	InstINVALID,
/* 1CA */	InstINVALID,
/* 1CB */	InstINVALID,
/* 1CC */	InstINVALID,
/* 1CD */	InstINVALID,
/* 1CE */	InstINVALID,
/* 1CF */	InstINVALID,
/* 1D0 */	InstINVALID,
/* 1D1 */	InstINVALID,
/* 1D2 */	InstINVALID,
/* 1D3 */	InstINVALID,
/* 1D4 */	InstINVALID,
/* 1D5 */	InstINVALID,
/* 1D6 */	InstINVALID,
/* 1D7 */	InstINVALID,
/* 1D8 */	InstINVALID,
/* 1D9 */	InstINVALID,
/* 1DA */	InstINVALID,
/* 1DB */	InstINVALID,
/* 1DC */	InstINVALID,
/* 1DD */	InstINVALID,
/* 1DE */	InstINVALID,
/* 1DF */	InstINVALID,
/* 1E0 */	InstINVALID,
/* 1E1 */	InstINVALID,
/* 1E2 */	InstINVALID,
/* 1E3 */	InstINVALID,
/* 1E4 */	InstINVALID,
/* 1E5 */	InstINVALID,
/* 1E6 */	InstINVALID,
/* 1E7 */	InstINVALID,
/* 1E8 */	InstINVALID,
/* 1E9 */	InstINVALID,
/* 1EA */	InstINVALID,
/* 1EB */	InstINVALID,
/* 1EC */	InstINVALID,
/* 1ED */	InstINVALID,
/* 1EE */	InstINVALID,
/* 1EF */	InstINVALID,
/* 1F0 */	InstINVALID,
/* 1F1 */	InstINVALID,
/* 1F2 */	InstINVALID,
/* 1F3 */	InstINVALID,
/* 1F4 */	InstINVALID,
/* 1F5 */	InstINVALID,
/* 1F6 */	InstINVALID,
/* 1F7 */	InstINVALID,
/* 1F8 */	InstINVALID,
/* 1F9 */	InstINVALID,
/* 1FA */	InstINVALID,
/* 1FB */	InstINVALID,
/* 1FC */	InstINVALID,
/* 1FD */	InstINVALID,
/* 1FE */	InstINVALID,
/* 1FF */	InstINVALID,
/* 200 */	InstINVALID,
/* 201 */	InstINVALID,
/* 202 */	InstINVALID,
/* 203 */	InstINVALID,
/* 204 */	InstINVALID,
/* 205 */	InstINVALID,
/* 206 */	InstINVALID,
/* 207 */	InstINVALID,
/* 208 */	InstINVALID,
/* 209 */	InstINVALID,
/* 20A */	InstINVALID,
/* 20B */	InstINVALID,
/* 20C */	InstINVALID,
/* 20D */	InstINVALID,
/* 20E */	InstINVALID,
/* 20F */	InstINVALID,
/* 210 */	InstINVALID,
/* 211 */	InstINVALID,
/* 212 */	InstINVALID,
/* 213 */	InstINVALID,
/* 214 */	InstINVALID,
/* 215 */	InstINVALID,
/* 216 */	InstINVALID,
/* 217 */	InstINVALID,
/* 218 */	InstINVALID,
/* 219 */	InstINVALID,
/* 21A */	InstINVALID,
/* 21B */	InstINVALID,
/* 21C */	InstINVALID,
/* 21D */	InstINVALID,
/* 21E */	InstINVALID,
/* 21F */	InstINVALID,
/* 220 */	InstINVALID,
/* 221 */	InstINVALID,
/* 222 */	InstINVALID,
/* 223 */	InstINVALID,
/* 224 */	InstINVALID,
/* 225 */	InstINVALID,
/* 226 */	InstINVALID,
/* 227 */	InstINVALID,
/* 228 */	InstINVALID,
/* 229 */	InstINVALID,
/* 22A */	InstINVALID,
/* 22B */	InstINVALID,
/* 22C */	InstINVALID,
/* 22D */	InstINVALID,
/* 22E */	InstINVALID,
/* 22F */	InstINVALID,
/* 230 */	InstINVALID,
/* 231 */	InstINVALID,
/* 232 */	InstINVALID,
/* 233 */	InstINVALID,
/* 234 */	InstINVALID,
/* 235 */	InstINVALID,
/* 236 */	InstINVALID,
/* 237 */	InstINVALID,
/* 238 */	InstINVALID,
/* 239 */	InstINVALID,
/* 23A */	InstINVALID,
/* 23B */	InstINVALID,
/* 23C */	InstINVALID,
/* 23D */	InstINVALID,
/* 23E */	InstINVALID,
/* 23F */	InstINVALID,
/* 240 */	InstINVALID,
/* 241 */	InstINVALID,
/* 242 */	InstINVALID,
/* 243 */	InstINVALID,
/* 244 */	InstINVALID,
/* 245 */	InstINVALID,
/* 246 */	InstINVALID,
/* 247 */	InstINVALID,
/* 248 */	InstINVALID,
/* 249 */	InstINVALID,
/* 24A */	InstINVALID,
/* 24B */	InstINVALID,
/* 24C */	InstINVALID,
/* 24D */	InstINVALID,
/* 24E */	InstINVALID,
/* 24F */	InstINVALID,
/* 250 */	InstINVALID,
/* 251 */	InstINVALID,
/* 252 */	InstINVALID,
/* 253 */	InstINVALID,
/* 254 */	InstINVALID,
/* 255 */	InstINVALID,
/* 256 */	InstINVALID,
/* 257 */	InstINVALID,
/* 258 */	InstINVALID,
/* 259 */	InstINVALID,
/* 25A */	InstINVALID,
/* 25B */	InstINVALID,
/* 25C */	InstINVALID,
/* 25D */	InstINVALID,
/* 25E */	InstINVALID,
/* 25F */	InstINVALID,
/* 260 */	InstINVALID,
/* 261 */	InstINVALID,
/* 262 */	InstINVALID,
/* 263 */	InstINVALID,
/* 264 */	InstINVALID,
/* 265 */	InstINVALID,
/* 266 */	InstINVALID,
/* 267 */	InstINVALID,
/* 268 */	InstINVALID,
/* 269 */	InstINVALID,
/* 26A */	InstINVALID,
/* 26B */	InstINVALID,
/* 26C */	InstINVALID,
/* 26D */	InstINVALID,
/* 26E */	InstINVALID,
/* 26F */	InstINVALID,
/* 270 */	InstINVALID,
/* 271 */	InstINVALID,
/* 272 */	InstINVALID,
/* 273 */	InstINVALID,
/* 274 */	InstINVALID,
/* 275 */	InstINVALID,
/* 276 */	InstINVALID,
/* 277 */	InstINVALID,
/* 278 */	InstINVALID,
/* 279 */	InstINVALID,
/* 27A */	InstINVALID,
/* 27B */	InstINVALID,
/* 27C */	InstINVALID,
/* 27D */	InstINVALID,
/* 27E */	InstINVALID,
/* 27F */	InstINVALID,
/* 280 */	InstINVALID,
/* 281 */	InstINVALID,
/* 282 */	InstINVALID,
/* 283 */	InstINVALID,
/* 284 */	InstINVALID,
/* 285 */	InstINVALID,
/* 286 */	InstINVALID,
/* 287 */	InstINVALID,
/* 288 */	InstINVALID,
/* 289 */	InstINVALID,
/* 28A */	InstINVALID,
/* 28B */	InstINVALID,
/* 28C */	InstINVALID,
/* 28D */	InstINVALID,
/* 28E */	InstINVALID,
/* 28F */	InstINVALID,
/* 290 */	InstINVALID,
/* 291 */	InstINVALID,
/* 292 */	InstINVALID,
/* 293 */	InstINVALID,
/* 294 */	InstINVALID,
/* 295 */	InstINVALID,
/* 296 */	InstINVALID,
/* 297 */	InstINVALID,
/* 298 */	InstINVALID,
/* 299 */	InstINVALID,
/* 29A */	InstINVALID,
/* 29B */	InstINVALID,
/* 29C */	InstINVALID,
/* 29D */	InstINVALID,
/* 29E */	InstINVALID,
/* 29F */	InstINVALID,
/* 2A0 */	InstINVALID,
/* 2A1 */	InstINVALID,
/* 2A2 */	InstINVALID,
/* 2A3 */	InstINVALID,
/* 2A4 */	InstINVALID,
/* 2A5 */	InstINVALID,
/* 2A6 */	InstINVALID,
/* 2A7 */	InstINVALID,
/* 2A8 */	InstINVALID,
/* 2A9 */	InstINVALID,
/* 2AA */	InstINVALID,
/* 2AB */	InstINVALID,
/* 2AC */	InstINVALID,
/* 2AD */	InstINVALID,
/* 2AE */	InstINVALID,
/* 2AF */	InstINVALID,
/* 2B0 */	InstINVALID,
/* 2B1 */	InstINVALID,
/* 2B2 */	InstINVALID,
/* 2B3 */	InstINVALID,
/* 2B4 */	InstINVALID,
/* 2B5 */	InstINVALID,
/* 2B6 */	InstINVALID,
/* 2B7 */	InstINVALID,
/* 2B8 */	InstINVALID,
/* 2B9 */	InstINVALID,
/* 2BA */	InstINVALID,
/* 2BB */	InstINVALID,
/* 2BC */	InstINVALID,
/* 2BD */	InstINVALID,
/* 2BE */	InstINVALID,
/* 2BF */	InstINVALID,
/* 2C0 */	InstINVALID,
/* 2C1 */	InstINVALID,
/* 2C2 */	InstINVALID,
/* 2C3 */	InstINVALID,
/* 2C4 */	InstINVALID,
/* 2C5 */	InstINVALID,
/* 2C6 */	InstINVALID,
/* 2C7 */	InstINVALID,
/* 2C8 */	InstINVALID,
/* 2C9 */	InstINVALID,
/* 2CA */	InstINVALID,
/* 2CB */	InstINVALID,
/* 2CC */	InstINVALID,
/* 2CD */	InstINVALID,
/* 2CE */	InstINVALID,
/* 2CF */	InstINVALID,
/* 2D0 */	InstINVALID,
/* 2D1 */	InstINVALID,
/* 2D2 */	InstINVALID,
/* 2D3 */	InstINVALID,
/* 2D4 */	InstINVALID,
/* 2D5 */	InstINVALID,
/* 2D6 */	InstINVALID,
/* 2D7 */	InstINVALID,
/* 2D8 */	InstINVALID,
/* 2D9 */	InstINVALID,
/* 2DA */	InstINVALID,
/* 2DB */	InstINVALID,
/* 2DC */	InstINVALID,
/* 2DD */	InstINVALID,
/* 2DE */	InstINVALID,
/* 2DF */	InstINVALID,
/* 2E0 */	InstINVALID,
/* 2E1 */	InstINVALID,
/* 2E2 */	InstINVALID,
/* 2E3 */	InstINVALID,
/* 2E4 */	InstINVALID,
/* 2E5 */	InstINVALID,
/* 2E6 */	InstINVALID,
/* 2E7 */	InstINVALID,
/* 2E8 */	InstINVALID,
/* 2E9 */	InstINVALID,
/* 2EA */	InstINVALID,
/* 2EB */	InstINVALID,
/* 2EC */	InstINVALID,
/* 2ED */	InstINVALID,
/* 2EE */	InstINVALID,
/* 2EF */	InstINVALID,
/* 2F0 */	InstINVALID,
/* 2F1 */	InstINVALID,
/* 2F2 */	InstINVALID,
/* 2F3 */	InstINVALID,
/* 2F4 */	InstINVALID,
/* 2F5 */	InstINVALID,
/* 2F6 */	InstINVALID,
/* 2F7 */	InstINVALID,
/* 2F8 */	InstINVALID,
/* 2F9 */	InstINVALID,
/* 2FA */	InstINVALID,
/* 2FB */	InstINVALID,
/* 2FC */	InstINVALID,
/* 2FD */	InstINVALID,
/* 2FE */	InstINVALID,
/* 2FF */	InstINVALID,
/* 300 */	InstINVALID,
/* 301 */	InstINVALID,
/* 302 */	InstINVALID,
/* 303 */	InstINVALID,
/* 304 */	InstINVALID,
/* 305 */	InstINVALID,
/* 306 */	InstINVALID,
/* 307 */	InstINVALID,
/* 308 */	InstINVALID,
/* 309 */	InstINVALID,
/* 30A */	InstINVALID,
/* 30B */	InstINVALID,
/* 30C */	InstINVALID,
/* 30D */	InstINVALID,
/* 30E */	InstINVALID,
/* 30F */	InstINVALID,
/* 310 */	InstINVALID,
/* 311 */	InstINVALID,
/* 312 */	InstINVALID,
/* 313 */	InstINVALID,
/* 314 */	InstINVALID,
/* 315 */	InstINVALID,
/* 316 */	InstINVALID,
/* 317 */	InstINVALID,
/* 318 */	InstINVALID,
/* 319 */	InstINVALID,
/* 31A */	InstINVALID,
/* 31B */	InstINVALID,
/* 31C */	InstINVALID,
/* 31D */	InstINVALID,
/* 31E */	InstINVALID,
/* 31F */	InstINVALID,
/* 320 */	InstINVALID,
/* 321 */	InstINVALID,
/* 322 */	InstINVALID,
/* 323 */	InstINVALID,
/* 324 */	InstINVALID,
/* 325 */	InstINVALID,
/* 326 */	InstINVALID,
/* 327 */	InstINVALID,
/* 328 */	InstINVALID,
/* 329 */	InstINVALID,
/* 32A */	InstINVALID,
/* 32B */	InstINVALID,
/* 32C */	InstINVALID,
/* 32D */	InstINVALID,
/* 32E */	InstINVALID,
/* 32F */	InstINVALID,
/* 330 */	InstINVALID,
/* 331 */	InstINVALID,
/* 332 */	InstINVALID,
/* 333 */	InstINVALID,
/* 334 */	InstINVALID,
/* 335 */	InstINVALID,
/* 336 */	InstINVALID,
/* 337 */	InstINVALID,
/* 338 */	InstINVALID,
/* 339 */	InstINVALID,
/* 33A */	InstINVALID,
/* 33B */	InstINVALID,
/* 33C */	InstINVALID,
/* 33D */	InstINVALID,
/* 33E */	InstINVALID,
/* 33F */	InstINVALID,
/* 340 */	InstINVALID,
/* 341 */	InstINVALID,
/* 342 */	InstINVALID,
/* 343 */	InstINVALID,
/* 344 */	InstINVALID,
/* 345 */	InstINVALID,
/* 346 */	InstINVALID,
/* 347 */	InstINVALID,
/* 348 */	InstINVALID,
/* 349 */	InstINVALID,
/* 34A */	InstINVALID,
/* 34B */	InstINVALID,
/* 34C */	InstINVALID,
/* 34D */	InstINVALID,
/* 34E */	InstINVALID,
/* 34F */	InstINVALID,
/* 350 */	InstINVALID,
/* 351 */	InstINVALID,
/* 352 */	InstINVALID,
/* 353 */	InstINVALID,
/* 354 */	InstINVALID,
/* 355 */	InstINVALID,
/* 356 */	InstINVALID,
/* 357 */	InstINVALID,
/* 358 */	InstINVALID,
/* 359 */	InstINVALID,
/* 35A */	InstINVALID,
/* 35B */	InstINVALID,
/* 35C */	InstINVALID,
/* 35D */	InstINVALID,
/* 35E */	InstINVALID,
/* 35F */	InstINVALID,
/* 360 */	InstINVALID,
/* 361 */	InstINVALID,
/* 362 */	InstINVALID,
/* 363 */	InstINVALID,
/* 364 */	InstINVALID,
/* 365 */	InstINVALID,
/* 366 */	InstINVALID,
/* 367 */	InstINVALID,
/* 368 */	InstINVALID,
/* 369 */	InstINVALID,
/* 36A */	InstINVALID,
/* 36B */	InstINVALID,
/* 36C */	InstINVALID,
/* 36D */	InstINVALID,
/* 36E */	InstINVALID,
/* 36F */	InstINVALID,
/* 370 */	InstINVALID,
/* 371 */	InstINVALID,
/* 372 */	InstINVALID,
/* 373 */	InstINVALID,
/* 374 */	InstINVALID,
/* 375 */	InstINVALID,
/* 376 */	InstINVALID,
/* 377 */	InstINVALID,
/* 378 */	InstINVALID,
/* 379 */	InstINVALID,
/* 37A */	InstINVALID,
/* 37B */	InstINVALID,
/* 37C */	InstINVALID,
/* 37D */	InstINVALID,
/* 37E */	InstINVALID,
/* 37F */	InstINVALID,
/* 380 */	InstINVALID,
/* 381 */	InstINVALID,
/* 382 */	InstINVALID,
/* 383 */	InstINVALID,
/* 384 */	InstINVALID,
/* 385 */	InstINVALID,
/* 386 */	InstINVALID,
/* 387 */	InstINVALID,
/* 388 */	InstINVALID,
/* 389 */	InstINVALID,
/* 38A */	InstINVALID,
/* 38B */	InstINVALID,
/* 38C */	InstINVALID,
/* 38D */	InstINVALID,
/* 38E */	InstINVALID,
/* 38F */	InstINVALID,
/* 390 */	InstINVALID,
/* 391 */	InstINVALID,
/* 392 */	InstINVALID,
/* 393 */	InstINVALID,
/* 394 */	InstINVALID,
/* 395 */	InstINVALID,
/* 396 */	InstINVALID,
/* 397 */	InstINVALID,
/* 398 */	InstINVALID,
/* 399 */	InstINVALID,
/* 39A */	InstINVALID,
/* 39B */	InstINVALID,
/* 39C */	InstINVALID,
/* 39D */	InstINVALID,
/* 39E */	InstINVALID,
/* 39F */	InstINVALID,
/* 3A0 */	InstINVALID,
/* 3A1 */	InstINVALID,
/* 3A2 */	InstINVALID,
/* 3A3 */	InstINVALID,
/* 3A4 */	InstINVALID,
/* 3A5 */	InstINVALID,
/* 3A6 */	InstINVALID,
/* 3A7 */	InstINVALID,
/* 3A8 */	InstINVALID,
/* 3A9 */	InstINVALID,
/* 3AA */	InstINVALID,
/* 3AB */	InstINVALID,
/* 3AC */	InstINVALID,
/* 3AD */	InstINVALID,
/* 3AE */	InstINVALID,
/* 3AF */	InstINVALID,
/* 3B0 */	InstINVALID,
/* 3B1 */	InstINVALID,
/* 3B2 */	InstINVALID,
/* 3B3 */	InstINVALID,
/* 3B4 */	InstINVALID,
/* 3B5 */	InstINVALID,
/* 3B6 */	InstINVALID,
/* 3B7 */	InstINVALID,
/* 3B8 */	InstINVALID,
/* 3B9 */	InstINVALID,
/* 3BA */	InstINVALID,
/* 3BB */	InstINVALID,
/* 3BC */	InstINVALID,
/* 3BD */	InstINVALID,
/* 3BE */	InstINVALID,
/* 3BF */	InstINVALID,
/* 3C0 */	InstINVALID,
/* 3C1 */	InstINVALID,
/* 3C2 */	InstINVALID,
/* 3C3 */	InstINVALID,
/* 3C4 */	InstINVALID,
/* 3C5 */	InstINVALID,
/* 3C6 */	InstINVALID,
/* 3C7 */	InstINVALID,
/* 3C8 */	InstINVALID,
/* 3C9 */	InstINVALID,
/* 3CA */	InstINVALID,
/* 3CB */	InstINVALID,
/* 3CC */	InstINVALID,
/* 3CD */	InstINVALID,
/* 3CE */	InstINVALID,
/* 3CF */	InstINVALID,
/* 3D0 */	InstINVALID,
/* 3D1 */	InstINVALID,
/* 3D2 */	InstINVALID,
/* 3D3 */	InstINVALID,
/* 3D4 */	InstINVALID,
/* 3D5 */	InstINVALID,
/* 3D6 */	InstINVALID,
/* 3D7 */	InstINVALID,
/* 3D8 */	InstINVALID,
/* 3D9 */	InstINVALID,
/* 3DA */	InstINVALID,
/* 3DB */	InstINVALID,
/* 3DC */	InstINVALID,
/* 3DD */	InstINVALID,
/* 3DE */	InstINVALID,
/* 3DF */	InstINVALID,
/* 3E0 */	InstINVALID,
/* 3E1 */	InstINVALID,
/* 3E2 */	InstINVALID,
/* 3E3 */	InstINVALID,
/* 3E4 */	InstINVALID,
/* 3E5 */	InstINVALID,
/* 3E6 */	InstINVALID,
/* 3E7 */	InstINVALID,
/* 3E8 */	InstINVALID,
/* 3E9 */	InstINVALID,
/* 3EA */	InstINVALID,
/* 3EB */	InstINVALID,
/* 3EC */	InstINVALID,
/* 3ED */	InstINVALID,
/* 3EE */	InstINVALID,
/* 3EF */	InstINVALID,
/* 3F0 */	InstINVALID,
/* 3F1 */	InstINVALID,
/* 3F2 */	InstINVALID,
/* 3F3 */	InstINVALID,
/* 3F4 */	InstINVALID,
/* 3F5 */	InstINVALID,
/* 3F6 */	InstINVALID,
/* 3F7 */	InstINVALID,
/* 3F8 */	InstINVALID,
/* 3F9 */	InstINVALID,
/* 3FA */	InstINVALID,
/* 3FB */	InstINVALID,
/* 3FC */	InstINVALID,
/* 3FD */	InstINVALID,
/* 3FE */	InstINVALID,
/* 3FF */	InstINVALID
};
